@@ -58,43 +58,89 @@ COMMANDS['rsv'] = CommandPreset(
 
 KDJ_WEIGHT_K = 3.0
 KDJ_WEIGHT_D = 2.0
-KDJ_WEIGHT_BASE = KDJ_WEIGHT_D / KDJ_WEIGHT_K
-KDJ_WEIGHT_INCREASE = 1.0 / KDJ_WEIGHT_K
 
 
-def kd(series):
+def ewma(
+    array: np.ndarray,
+    period: int,
+    init: float
+):
+    """Exponentially weighted moving average
+    https://en.wikipedia.org/wiki/Moving_average#Exponential_moving_average
+
+    with
+    - init value as `50.0`
+    - alpha as `1 / period`
+    """
+
     # If there is no value k or value d of the previous day,
     # then use 50.0
-    k = 50.0
+    k = init
+    alpha = 1. / period
+    base = 1 - alpha
 
-    for i in series:
-        k = KDJ_WEIGHT_BASE * k + KDJ_WEIGHT_INCREASE * i
+    for i in array:
+        k = base * k + alpha * i
         yield k
 
 
-def kdj_k(df, s, period):
+def kdj_k(df, s, period_rsv, period_k, init):
     """Gets KDJ K
     """
 
-    rsv = df.exec(f'rsv:{period}')[s]
+    rsv = df.exec(f'rsv:{period_rsv}')[s]
 
-    return np.fromiter(kd(rsv), float), period
+    return np.fromiter(ewma(rsv, period_k, init), float), period_rsv
 
 
-def kdj_d(df, s, period):
-    k = df.exec(f'kdj.k:{period}')[s]
+def kdj_d(df, s, period_rsv, period_k, period_d, init):
+    k = df.exec(f'kdj.k:{period_rsv},{period_k},{init}')[s]
 
-    return np.fromiter(kd(k), float), period
+    return np.fromiter(ewma(k, period_d, init), float), period_rsv
 
 
-def kdj_j(df, s, period):
-    k = df.exec(f'kdj.k:{period}')[s]
-    d = df.exec(f'kdj.d:{period}')[s]
+def kdj_j(df, s, period_rsv, period_k, period_d, init):
+    k = df.exec(f'kdj.k:{period_rsv},{period_k},{init}')[s]
+    d = df.exec(f'kdj.d:{period_rsv},{period_k},{period_d},{init}')[s]
 
-    return KDJ_WEIGHT_K * k - KDJ_WEIGHT_D * d, period
+    return KDJ_WEIGHT_K * k - KDJ_WEIGHT_D * d, period_rsv
 
 
-kdj_arg_period = [(9, period_to_int)]
+def init_to_float(value):
+    try:
+        value = float(value)
+    except Exception:
+        raise ValueError(
+            f'init_value must be a float, but got `{value}`'
+        )
+
+    if value < 0. or value > 100.:
+        raise ValueError(
+            f'init_value must be in between 0 and 100, but got `{value}`'
+        )
+
+    return value
+
+
+arg_period_k = (3, period_to_int)
+kdj_common_args = [
+    (9, period_to_int),
+    arg_period_k
+]
+arg_init = (50., init_to_float)
+
+args_k = [
+    *kdj_common_args,
+    arg_init
+]
+
+# The default args for KDJ is 9, 3, 3, 50.
+args_dj = [
+    *kdj_common_args,
+    arg_period_k,
+    arg_init
+]
+
 
 COMMANDS['kdj'] = CommandPreset(
     # We must specify sub command for kdj, such as
@@ -102,17 +148,17 @@ COMMANDS['kdj'] = CommandPreset(
     subs_map=dict(
         k=CommandPreset(
             kdj_k,
-            kdj_arg_period
+            args_k
         ),
 
         d=CommandPreset(
             kdj_d,
-            kdj_arg_period
+            args_dj
         ),
 
         j=CommandPreset(
             kdj_j,
-            kdj_arg_period
+            args_dj
         )
     )
 )
