@@ -76,13 +76,23 @@ KEY_DIRECTIVES_CACHE = '__stock_directives_cache'
 
 
 def copy_stock_metas(source, target):
-    columns = source.columns
+    columns = target.columns
+
+    # If the new dataframe has been truncated,
+    # Then we need to clean the column info
+
+    # We just set the size of the info to zero to avoid complexity
+    need_clean = len(target) < len(source)
 
     source_aliases_map = getattr(source, KEY_ALIAS_MAP, None)
 
     if source_aliases_map is not None:
         aliases_map = {}
         for alias, column in source_aliases_map.items():
+            # Column `column` might be dropped in `target`
+            # by dataframe.drop(columns=some_columns)
+            # so we need to check it
+
             # TODO: if alias is in columns, something wrong happened
             # - support .iloc, loc, and other indexing and setting methods
             if column in columns:
@@ -98,7 +108,12 @@ def copy_stock_metas(source, target):
         columns_info_map = {}
         for column, info in source_columns_info_map.items():
             if column in columns:
-                columns_info_map[column] = info
+
+                # Set the size to 0,
+                # which indicates that the column needs to be calculated again
+                columns_info_map[
+                    column
+                ] = info.update(0) if need_clean else info
 
         object.__setattr__(target, KEY_COLUMNS_INFO_MAP, columns_info_map)
 
@@ -203,6 +218,15 @@ def rolling_calc(
     fill=np.nan,
     stride: int = 8
 ) -> np.ndarray:
+    """Creates a `period`-period rolling window and apply
+    `func` to the items
+    """
+
+    length = len(array)
+
+    if period > length:
+        return np.repeat(fill, length)
+
     unshifted = np.apply_along_axis(
         func,
         1,
