@@ -1,18 +1,13 @@
-
-from pathlib import Path
-
 import pytest
-import pandas as pd
 import numpy as np
 
 from stock_pandas import StockDataFrame
-
-csv = (Path(__file__).parent.parent / 'example' / 'tencent.csv').resolve()
+from .common import get_tencent
 
 
 @pytest.fixture
 def stock():
-    return StockDataFrame(pd.read_csv(csv), date_column='time_key')
+    return get_tencent()
 
 
 def test_append(stock):
@@ -106,3 +101,12 @@ def test_rsi(stock):
 def test_bbi(stock):
     assert stock['repeat:(bbi < column:close),8']['2020-02-13']
     assert not stock['repeat:(bbi < column:close),9']['2020-02-13']
+
+
+def test_kdj(stock):
+    assert stock['kdj.j']['2020-02-07'] > 120
+    assert stock['kdj.j']['2020-01-31'] < -1
+    assert stock['kdj.j']['2020-02-24'] < -14
+
+    kdjj = stock['kdj.j']['2020-02-07']
+    assert stock['kdj.j:9,3,3,50']['2020-02-07'] == kdjj
